function Struct3DHF = Dynamics3DFollicle_Cut_NoConnect_eLife(StructStack)



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%     PARAMETERS     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%Production coefficients 
alphaW1 = 40;
alphaW0 = 3 ;
alphaB0 = -8;
alphaB1 = 20;




%Temporal discretizaion
dt = 0.01;

%Geomotry coefficients  
   %spacial discretization
   dx = 2;
   dy = 2;
   dz = 0.2;
    
   %computation domain [0, X]*[0, Y]*[-Z, 0]
   X = 100; 
   Y = 100; 
   Z = 10;
   dX = X/2; 
   
   NX = X/dx;
   NY = Y/dy;
   NZ = Z/dz;
 
   
   
   %region I: bulge [  h_I*dz,  h_I*dh + d_I*dz] 
   hI = - 5 ;
   dI = 3 ;
   %region II: sHG/Matrix + DP  [h(t), h(t) + d_II*dh]
   dII = 4;
   %max/min length of the follicle
   hmin = (hI - dII)*dz;
   hmax = - 30*dz;

%explicit finite difference, D*dt/(dx^2 + dy^2) < 1/2
   
%Error allowence in computation
err = 0; 


 
%Threshold of concentration differnece
DeltaT = 2.85*ones(NX+1,NY+1); 
 

%Diffusion coefficient  
DAct = 0.5;
DInh = 0.5;



%Reaction coefficients 
kActOn = 0.3;
kInhOn = 0.5;
kActOff = 0;
kInhOff = 0;

 
%Degradation coefficients
kActDeg = 2;
kInhDeg = 7; 
 

%Preservation of receptors
RActTot = 15; 
RInhTot = 15; 
 
betaAct = 4; 
 
betaInh = 2.3 ;  

 

%Noise
lambda = 0.02; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%  PREPARATION  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%  TIME  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 



HFTime = - ones(NX+1,NY+1);
%FollicleTime(i) : +/-t for the (i,j)-th follicle
%t is the time in the current phase: t - t^+ or t - t^-
%+/- gives the current time phase
 


%%%%%%%%%%%%%%%%%%%%%%%% CONCENTRATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 



%%%%%%%%%%%%%%%%%%%%%%% FOLLICLE LENGTH %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 

h = hmin*ones(NX+1,NY+1);





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



NX_Sq_L = 15;
NX_Sq_R = 36;
NY_Sq_B = 28;
NY_Sq_T = 30;

 


NStep = 30000 ; 
 

 
 
    %concentration difference
    AveActLR = zeros(NX+1,NY+1);
    AveInhLR = zeros(NX+1,NY+1);
    for nstack = - hI + 1 - dI:1:- hI
        
        AveActLR = AveActLR + StructStack(nstack).ActLR;
        AveInhLR = AveInhLR + StructStack(nstack).InhLR;
                
    end
    AveActLR = AveActLR/dI;
    AveInhLR = AveInhLR/dI;
    Conc3DDiff = AveActLR - AveInhLR;
    
    
 
Struct3DHF(1).AveActLR = AveActLR;
Struct3DHF(1).AveInhLR = AveInhLR;
Struct3DHF(1).h = h;
Struct3DHF(1).HFTime = HFTime;
Struct3DHF(1).Time = 0;



CountStep = 1;
 
 
 

 
cm = jet(8);

Thrd2=0;

for nstep = 1:1:NStep
    
 
     
    %Inh production rate
    InhProd = alphaB0 + alphaB1*( -h  + hmin + dI*dz);    
    

    
 
        
        
    for nxHF = 1:1:NX+1
        for nyHF = 1:1:NY+1
            
            %Act production rate
            if HFTime(nxHF,nyHF) > 0
                %phase+
                ActProd(nxHF,nyHF) = alphaW1*( - hmax + h(nxHF,nyHF) )/(- hmax + hmin) ; 
            else
                ActProd(nxHF,nyHF) = alphaW0;
            end
               
 
            if nstep <= 10 && nyHF == NY + 1
                
                ActProd(nxHF,nyHF) = ActProd(nxHF,nyHF) + 400;
                
            end
    
 
            
            
            %HF length
            h(nxHF,nyHF) = De3DFollicleLength(h(nxHF,nyHF), HFTime(nxHF,nyHF), dt, hmax, hmin);
       
            
            %HF phase 
            if h(nxHF,nyHF) <= hmin - dz && Conc3DDiff(nxHF,nyHF) < Thrd2 && HFTime(nxHF,nyHF) > 0  
                %time phase turns from + to -
                HFTime(nxHF,nyHF) = -1;

            end
    
            if h(nxHF,nyHF) >= hmin - err && Conc3DDiff(nxHF,nyHF) >= DeltaT(nxHF,nyHF) && HFTime(nxHF,nyHF) <0
                %starts a new cycle, i.e., time phase turns from - to +
                HFTime(nxHF,nyHF) = 1;
                DeltaT(nxHF,nyHF) = 2.6; 
            end
            
            
            %No HF in this region
            if nyHF >= NY_Sq_B && nyHF <= NY_Sq_T && nxHF >= NX_Sq_L && nxHF <= NX_Sq_R
                h(nxHF,nyHF) = 0;
                HFTime(nxHF,nyHF) = 0;
                
                
            end
            
 
        end
    end
       
  

        

    HFTime = HFTime + sign(HFTime); 
    
 
    
  
   for nstack = 1:1:NZ+1
        
        %diffusion in the z direction
        if nstack == 1
            %top, d (ActL/InhL) / dz = 0
            DeActL = DAct*(2*StructStack(2).ActL - 2*StructStack(1).ActL)/dz^2;
            DeInhL = DInh*(2*StructStack(2).InhL - 2*StructStack(1).InhL)/dz^2;
        elseif nstack == NZ+1
            %bottom, ActL/InhL = 0
            DeActL = DAct*(StructStack(NZ).ActL - 2*StructStack(NZ+1).ActL)/dz^2;
            DeInhL = DInh*(StructStack(NZ).InhL - 2*StructStack(NZ+1).InhL)/dz^2;
        else       
            DeActL = DAct*(StructStack(nstack-1).ActL - 2*StructStack(nstack).ActL + StructStack(nstack+1).ActL)/dz^2;
            DeInhL = DInh*(StructStack(nstack-1).InhL - 2*StructStack(nstack).InhL + StructStack(nstack+1).InhL)/dz^2;
        end
        
        
        
        ActL = StructStack(nstack).ActL;
        InhL = StructStack(nstack).InhL;
        
        
        %Domain
        
        DeActL(2:NX,2:NY_Sq_B-1) = DeActL(2:NX,2:NY_Sq_B-1) + DAct*(( ActL(1:NX-1,2:NY_Sq_B-1) - 2*ActL(2:NX,2:NY_Sq_B-1) + ActL(3:NX+1,2:NY_Sq_B-1) )/dx^2 + ( ActL(2:NX,1:NY_Sq_B-2) - 2*ActL(2:NX,2:NY_Sq_B-1) + ActL(2:NX,3:NY_Sq_B) )/dy^2);
        DeInhL(2:NX,2:NY_Sq_B-1) = DeInhL(2:NX,2:NY_Sq_B-1) + DInh*(( InhL(1:NX-1,2:NY_Sq_B-1) - 2*InhL(2:NX,2:NY_Sq_B-1) + InhL(3:NX+1,2:NY_Sq_B-1) )/dx^2 + ( InhL(2:NX,1:NY_Sq_B-2) - 2*InhL(2:NX,2:NY_Sq_B-1) + InhL(2:NX,3:NY_Sq_B) )/dy^2);
        
        DeActL(2:NX,NY_Sq_T+1:NY) = DeActL(2:NX,NY_Sq_T+1:NY) + DAct*(( ActL(1:NX-1,NY_Sq_T+1:NY) - 2*ActL(2:NX,NY_Sq_T+1:NY) + ActL(3:NX+1,NY_Sq_T+1:NY) )/dx^2 + ( ActL(2:NX,NY_Sq_T:NY-1) - 2*ActL(2:NX,NY_Sq_T+1:NY) + ActL(2:NX,NY_Sq_T+2:NY+1) )/dy^2);
        DeInhL(2:NX,NY_Sq_T+1:NY) = DeInhL(2:NX,NY_Sq_T+1:NY) + DInh*(( InhL(1:NX-1,NY_Sq_T+1:NY) - 2*InhL(2:NX,NY_Sq_T+1:NY) + InhL(3:NX+1,NY_Sq_T+1:NY) )/dx^2 + ( InhL(2:NX,NY_Sq_T:NY-1) - 2*InhL(2:NX,NY_Sq_T+1:NY) + InhL(2:NX,NY_Sq_T+2:NY+1) )/dy^2);
        
        DeActL(2:NX_Sq_L-1,NY_Sq_B:NY_Sq_T) = DeActL(2:NX_Sq_L-1,NY_Sq_B:NY_Sq_T) + DAct*(( ActL(1:NX_Sq_L-2,NY_Sq_B:NY_Sq_T) - 2*ActL(2:NX_Sq_L-1,NY_Sq_B:NY_Sq_T) + ActL(3:NX_Sq_L,NY_Sq_B:NY_Sq_T) )/dx^2 + ( ActL(2:NX_Sq_L-1,NY_Sq_B-1:NY_Sq_T-1) - 2*ActL(2:NX_Sq_L-1,NY_Sq_B:NY_Sq_T) + ActL(2:NX_Sq_L-1,NY_Sq_B+1:NY_Sq_T+1) )/dy^2);
        DeInhL(2:NX_Sq_L-1,NY_Sq_B:NY_Sq_T) = DeInhL(2:NX_Sq_L-1,NY_Sq_B:NY_Sq_T) + DInh*(( InhL(1:NX_Sq_L-2,NY_Sq_B:NY_Sq_T) - 2*InhL(2:NX_Sq_L-1,NY_Sq_B:NY_Sq_T) + InhL(3:NX_Sq_L,NY_Sq_B:NY_Sq_T) )/dx^2 + ( InhL(2:NX_Sq_L-1,NY_Sq_B-1:NY_Sq_T-1) - 2*InhL(2:NX_Sq_L-1,NY_Sq_B:NY_Sq_T) + InhL(2:NX_Sq_L-1,NY_Sq_B+1:NY_Sq_T+1) )/dy^2);
        
        DeActL(NX_Sq_L+1:NX,NY_Sq_B:NY_Sq_T) = DeActL(NX_Sq_L+1:NX,NY_Sq_B:NY_Sq_T) + DAct*(( ActL(NX_Sq_L:NX-1,NY_Sq_B:NY_Sq_T) - 2*ActL(NX_Sq_L+1:NX,NY_Sq_B:NY_Sq_T) + ActL(NX_Sq_L+2:NX+1,NY_Sq_B:NY_Sq_T) )/dx^2 + ( ActL(NX_Sq_L+1:NX,NY_Sq_B-1:NY_Sq_T-1) - 2*ActL(NX_Sq_L+1:NX,NY_Sq_B:NY_Sq_T) + ActL(NX_Sq_L+1:NX,NY_Sq_B+1:NY_Sq_T+1) )/dy^2);
        DeInhL(NX_Sq_L+1:NX,NY_Sq_B:NY_Sq_T) = DeInhL(NX_Sq_L+1:NX,NY_Sq_B:NY_Sq_T) + DInh*(( InhL(NX_Sq_L:NX-1,NY_Sq_B:NY_Sq_T) - 2*InhL(NX_Sq_L+1:NX,NY_Sq_B:NY_Sq_T) + InhL(NX_Sq_L+2:NX+1,NY_Sq_B:NY_Sq_T) )/dx^2 + ( InhL(NX_Sq_L+1:NX,NY_Sq_B-1:NY_Sq_T-1) - 2*InhL(NX_Sq_L+1:NX,NY_Sq_B:NY_Sq_T) + InhL(NX_Sq_L+1:NX,NY_Sq_B+1:NY_Sq_T+1) )/dy^2);
        
        
        %no leak
        %Corner
        
        DeActL(1,1) = DeActL(1,1) + DAct*((2*ActL(2,1) - 2*ActL(1,1))/dx^2 + (2*ActL(1,2) - 2*ActL(1,1))/dy^2);
        DeInhL(1,1) = DeInhL(1,1) + DInh*((2*InhL(2,1) - 2*InhL(1,1))/dx^2 + (2*InhL(1,2) - 2*InhL(1,1))/dy^2);
        
        DeActL(NX+1,1) = DeActL(NX+1,1) + DAct*((2*ActL(NX,1) - 2*ActL(NX+1,1))/dx^2 + (2*ActL(NX+1,2) - 2*ActL(NX+1,1))/dy^2);
        DeInhL(NX+1,1) = DeInhL(NX+1,1) + DInh*((2*InhL(NX,1) - 2*InhL(NX+1,1))/dx^2 + (2*InhL(NX+1,2) - 2*InhL(NX+1,1))/dy^2);
        
        DeActL(1,NY+1) = DeActL(1,NY+1) + DAct*((2*ActL(1,NY) - 2*ActL(1,NY+1))/dy^2 + (2*ActL(2,NY+1) - 2*ActL(1,NY+1))/dx^2);
        DeInhL(1,NY+1) = DeInhL(1,NY+1) + DInh*((2*InhL(1,NY) - 2*InhL(1,NY+1))/dy^2 + (2*InhL(2,NY+1) - 2*InhL(1,NY+1))/dx^2);
        
        DeActL(NX+1,NY+1) = DeActL(NX+1,NY+1) + DAct*((2*ActL(NX,NY+1) - 2*ActL(NX+1,NY+1))/dx^2 + (2*ActL(NX+1,NY) - 2*ActL(NX+1,NY+1))/dy^2);
        DeInhL(NX+1,NY+1) = DeInhL(NX+1,NY+1) + DInh*((2*InhL(NX,NY+1) - 2*InhL(NX+1,NY+1))/dx^2 + (2*InhL(NX+1,NY) - 2*InhL(NX+1,NY+1))/dy^2);
        
 
        DeActL(NX_Sq_L,NY_Sq_B) = DeActL(NX_Sq_L,NY_Sq_B) + DAct*(( ActL(NX_Sq_L-1,NY_Sq_B) - 2*ActL(NX_Sq_L,NY_Sq_B) + ActL(NX_Sq_L+1,NY_Sq_B) )/dx^2 + ( ActL(NX_Sq_L,NY_Sq_B-1) - 2*ActL(NX_Sq_L,NY_Sq_B) + ActL(NX_Sq_L,NY_Sq_B+1) )/dy^2  );
        DeInhL(NX_Sq_L,NY_Sq_B) = DeInhL(NX_Sq_L,NY_Sq_B) + DInh*(( InhL(NX_Sq_L-1,NY_Sq_B) - 2*InhL(NX_Sq_L,NY_Sq_B) + InhL(NX_Sq_L+1,NY_Sq_B) )/dx^2 + ( InhL(NX_Sq_L,NY_Sq_B-1) - 2*InhL(NX_Sq_L,NY_Sq_B) + InhL(NX_Sq_L,NY_Sq_B+1) )/dy^2  );
        
        DeActL(NX_Sq_R,NY_Sq_B) = DeActL(NX_Sq_R,NY_Sq_B) + DAct*(( ActL(NX_Sq_R-1,NY_Sq_B) - 2*ActL(NX_Sq_R,NY_Sq_B) + ActL(NX_Sq_R+1,NY_Sq_B) )/dx^2 + ( ActL(NX_Sq_R,NY_Sq_B-1) - 2*ActL(NX_Sq_R,NY_Sq_B) + ActL(NX_Sq_R,NY_Sq_B+1) )/dy^2  );
        DeInhL(NX_Sq_R,NY_Sq_B) = DeInhL(NX_Sq_R,NY_Sq_B) + DInh*(( InhL(NX_Sq_R-1,NY_Sq_B) - 2*InhL(NX_Sq_R,NY_Sq_B) + InhL(NX_Sq_R+1,NY_Sq_B) )/dx^2 + ( InhL(NX_Sq_R,NY_Sq_B-1) - 2*InhL(NX_Sq_R,NY_Sq_B) + InhL(NX_Sq_R,NY_Sq_B+1) )/dy^2  );
        
        DeActL(NX_Sq_L,NY_Sq_T) = DeActL(NX_Sq_L,NY_Sq_T) + DAct*(( ActL(NX_Sq_L-1,NY_Sq_T) - 2*ActL(NX_Sq_L,NY_Sq_T) + ActL(NX_Sq_L+1,NY_Sq_T) )/dx^2 + ( ActL(NX_Sq_L,NY_Sq_T-1) - 2*ActL(NX_Sq_L,NY_Sq_T) + ActL(NX_Sq_L,NY_Sq_T+1) )/dy^2  );
        DeInhL(NX_Sq_L,NY_Sq_T) = DeInhL(NX_Sq_L,NY_Sq_T) + DInh*(( InhL(NX_Sq_L-1,NY_Sq_T) - 2*InhL(NX_Sq_L,NY_Sq_T) + InhL(NX_Sq_L+1,NY_Sq_T) )/dx^2 + ( InhL(NX_Sq_L,NY_Sq_T-1) - 2*InhL(NX_Sq_L,NY_Sq_T) + InhL(NX_Sq_L,NY_Sq_T+1) )/dy^2  );
        
        DeActL(NX_Sq_R,NY_Sq_T) = DeActL(NX_Sq_R,NY_Sq_T) + DAct*(( ActL(NX_Sq_R-1,NY_Sq_T) - 2*ActL(NX_Sq_R,NY_Sq_T) + ActL(NX_Sq_R+1,NY_Sq_T) )/dx^2 + ( ActL(NX_Sq_R,NY_Sq_T-1) - 2*ActL(NX_Sq_R,NY_Sq_T) + ActL(NX_Sq_R,NY_Sq_T+1) )/dy^2  );
        DeInhL(NX_Sq_R,NY_Sq_T) = DeInhL(NX_Sq_R,NY_Sq_T) + DInh*(( InhL(NX_Sq_R-1,NY_Sq_T) - 2*InhL(NX_Sq_R,NY_Sq_T) + InhL(NX_Sq_R+1,NY_Sq_T) )/dx^2 + ( InhL(NX_Sq_R,NY_Sq_T-1) - 2*InhL(NX_Sq_R,NY_Sq_T) + InhL(NX_Sq_R,NY_Sq_T+1) )/dy^2  );
        
        
        %Edge
        
         
        DeActL(1,2:NY) = DeActL(1,2:NY) + DAct*((2*ActL(2,2:NY) - 2*ActL(1,2:NY))/dx^2 + ( ActL(1,1:NY-1) - 2*ActL(1,2:NY) + ActL(1,3:NY+1) )/dy^2  );
        DeInhL(1,2:NY) = DeInhL(1,2:NY) + DInh*((2*InhL(2,2:NY) - 2*InhL(1,2:NY))/dx^2 + ( InhL(1,1:NY-1) - 2*InhL(1,2:NY) + InhL(1,3:NY+1) )/dy^2  );
        
        DeActL(NX+1,2:NY) = DeActL(NX+1,2:NY) + DAct*((2*ActL(NX,2:NY) - 2*ActL(NX+1,2:NY))/dx^2 + ( ActL(NX+1,1:NY-1) - 2*ActL(NX+1,2:NY) + ActL(NX+1,3:NY+1) )/dy^2  );
        DeInhL(NX+1,2:NY) = DeInhL(NX+1,2:NY) + DInh*((2*InhL(NX,2:NY) - 2*InhL(NX+1,2:NY))/dx^2 + ( InhL(NX+1,1:NY-1) - 2*InhL(NX+1,2:NY) + InhL(NX+1,3:NY+1) )/dy^2  );
        
        DeActL(2:NX,1) = DeActL(2:NX,1) + DAct*((2*ActL(2:NX,2) - 2*ActL(2:NX,1))/dy^2 + ( ActL(1:NX-1,1) - 2*ActL(2:NX,1) + ActL(3:NX+1,1) )/dx^2  );
        DeInhL(2:NX,1) = DeInhL(2:NX,1) + DInh*((2*InhL(2:NX,2) - 2*InhL(2:NX,1))/dy^2 + ( InhL(1:NX-1,1) - 2*InhL(2:NX,1) + InhL(3:NX+1,1) )/dx^2  );
        
        DeActL(2:NX,NY+1) = DeActL(2:NX,NY+1) + DAct*((2*ActL(2:NX,NY) - 2*ActL(2:NX,NY+1))/dy^2 + ( ActL(1:NX-1,NY+1) - 2*ActL(2:NX,NY+1) + ActL(3:NX+1,NY+1) )/dx^2  );
        DeInhL(2:NX,NY+1) = DeInhL(2:NX,NY+1) + DInh*((2*InhL(2:NX,NY) - 2*InhL(2:NX,NY+1))/dy^2 + ( InhL(1:NX-1,NY+1) - 2*InhL(2:NX,NY+1) + InhL(3:NX+1,NY+1) )/dx^2  );
        
        
        
        
        DeActL(NX_Sq_L,NY_Sq_B+1:NY_Sq_T-1) = DeActL(NX_Sq_L,NY_Sq_B+1:NY_Sq_T-1) + DAct*((2*ActL(NX_Sq_L-1,NY_Sq_B+1:NY_Sq_T-1) - 2*ActL(NX_Sq_L,NY_Sq_B+1:NY_Sq_T-1))/dx^2 + ( ActL(NX_Sq_L,NY_Sq_B:NY_Sq_T-2) - 2*ActL(NX_Sq_L,NY_Sq_B+1:NY_Sq_T-1) + ActL(NX_Sq_L,NY_Sq_B+2:NY_Sq_T) )/dy^2  );
        DeInhL(NX_Sq_L,NY_Sq_B+1:NY_Sq_T-1) = DeInhL(NX_Sq_L,NY_Sq_B+1:NY_Sq_T-1) + DInh*((2*InhL(NX_Sq_L-1,NY_Sq_B+1:NY_Sq_T-1) - 2*InhL(NX_Sq_L,NY_Sq_B+1:NY_Sq_T-1))/dx^2 + ( InhL(NX_Sq_L,NY_Sq_B:NY_Sq_T-2) - 2*InhL(NX_Sq_L,NY_Sq_B+1:NY_Sq_T-1) + InhL(NX_Sq_L,NY_Sq_B+2:NY_Sq_T) )/dy^2  );
        
        DeActL(NX_Sq_R,NY_Sq_B+1:NY_Sq_T-1) = DeActL(NX_Sq_R,NY_Sq_B+1:NY_Sq_T-1) + DAct*((2*ActL(NX_Sq_R+1,NY_Sq_B+1:NY_Sq_T-1) - 2*ActL(NX_Sq_R,NY_Sq_B+1:NY_Sq_T-1))/dx^2 + ( ActL(NX_Sq_R,NY_Sq_B:NY_Sq_T-2) - 2*ActL(NX_Sq_R,NY_Sq_B+1:NY_Sq_T-1) + ActL(NX_Sq_R,NY_Sq_B+2:NY_Sq_T) )/dy^2  );
        DeInhL(NX_Sq_R,NY_Sq_B+1:NY_Sq_T-1) = DeInhL(NX_Sq_R,NY_Sq_B+1:NY_Sq_T-1) + DInh*((2*InhL(NX_Sq_R+1,NY_Sq_B+1:NY_Sq_T-1) - 2*InhL(NX_Sq_R,NY_Sq_B+1:NY_Sq_T-1))/dx^2 + ( InhL(NX_Sq_R,NY_Sq_B:NY_Sq_T-2) - 2*InhL(NX_Sq_R,NY_Sq_B+1:NY_Sq_T-1) + InhL(NX_Sq_R,NY_Sq_B+2:NY_Sq_T) )/dy^2  );
        
        DeActL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_B) = DeActL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_B) + DAct*((2*ActL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_B-1) - 2*ActL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_B))/dy^2 + ( ActL(NX_Sq_L:NX_Sq_R-2,NY_Sq_B) - 2*ActL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_B) + ActL(NX_Sq_L+2:NX_Sq_R,NY_Sq_B) )/dx^2  );
        DeInhL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_B) = DeInhL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_B) + DInh*((2*InhL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_B-1) - 2*InhL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_B))/dy^2 + ( InhL(NX_Sq_L:NX_Sq_R-2,NY_Sq_B) - 2*InhL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_B) + InhL(NX_Sq_L+2:NX_Sq_R,NY_Sq_B) )/dx^2  );
        
        DeActL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_T) = DeActL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_T) + DAct*((2*ActL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_T+1) - 2*ActL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_T))/dy^2 + ( ActL(NX_Sq_L:NX_Sq_R-2,NY_Sq_T) - 2*ActL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_T) + ActL(NX_Sq_L+2:NX_Sq_R,NY_Sq_T) )/dx^2  );
        DeInhL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_T) = DeInhL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_T) + DInh*((2*InhL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_T+1) - 2*InhL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_T))/dy^2 + ( InhL(NX_Sq_L:NX_Sq_R-2,NY_Sq_T) - 2*InhL(NX_Sq_L+1:NX_Sq_R-1,NY_Sq_T) + InhL(NX_Sq_L+2:NX_Sq_R,NY_Sq_T) )/dx^2  );
        
        
        StructStack(nstack).DeActL = DeActL;
        StructStack(nstack).DeInhL = DeInhL;
 
        
        
    end
    
    for nstack = 1:1:NZ+1
        
        StructStack(nstack).ActL = StructStack(nstack).ActL + dt*(StructStack(nstack).DeActL);
        StructStack(nstack).InhL = StructStack(nstack).InhL + dt*(StructStack(nstack).DeInhL);
 
 
        
        %reaction, region I
        if nstack >= - hI - dI + 1 && nstack <= -hI
            
            ActL = StructStack(nstack).ActL;
            ActLR = StructStack(nstack).ActLR;
            InhL = StructStack(nstack).InhL;
            InhLR = StructStack(nstack).InhLR;
            
  
                
            StructStack(nstack).ActL = ActL + (- kActOn*ActL.*(RActTot - ActLR) + kActOff*ActLR)*dt;
            StructStack(nstack).InhL = InhL + (- kInhOn*InhL.*(RInhTot - InhLR) + kInhOff*InhLR)*dt;
            StructStack(nstack).ActLR = ActLR + (kActOn*ActL.*(RActTot - ActLR) - (kActOff + kActDeg)*ActLR + betaAct)*dt + lambda*sqrt( dt)*randn(NX+1,NY+1).*ActLR;
            StructStack(nstack).InhLR = InhLR + (kInhOn*InhL.*(RInhTot - InhLR) - (kInhOff + kInhDeg)*InhLR + betaInh)*dt + lambda*sqrt( dt)*randn(NX+1,NY+1).*InhLR;
            
            
            StructStack(nstack).ActLR = max(StructStack(nstack).ActLR,0 );
            StructStack(nstack).InhLR = max(StructStack(nstack).InhLR,0 );
            

            
                
 

        
        end
         
        
        %production, region II
        if nstack >= -hI  
        for nxHF = 1:1:NX+1
            for nyHF = 1:1:NY+1
                if nstack >= floor(- h(nxHF,nyHF)/dz) - dII + 1 && nstack <= ceil(- h(nxHF,nyHF)/dz)
                    
                    ActL =  StructStack(nstack).ActL;
                    ActL(nxHF,nyHF) = ActL(nxHF,nyHF) + ActProd(nxHF,nyHF)*dt;
                    StructStack(nstack).ActL = ActL;
                    
                    InhL =  StructStack(nstack).InhL;
                    InhL(nxHF,nyHF) = InhL(nxHF,nyHF) + InhProd(nxHF,nyHF)*dt;
                    
                    
                    StructStack(nstack).InhL = InhL;
                    
                    
                end
            end
            

        end
        end
        
        

        
        StructStack(nstack).ActL = max(StructStack(nstack).ActL ,0);
        StructStack(nstack).InhL = max(StructStack(nstack).InhL ,0);

        
        
    end

       
    
   
    %concentration difference
    AveActLR = zeros(NX+1,NY+1);
    AveInhLR = zeros(NX+1,NY+1);
    for nstack = - hI + 1 - dI:1:- hI
        
        AveActLR = AveActLR + StructStack(nstack).ActLR;
        AveInhLR = AveInhLR + StructStack(nstack).InhLR;
                
    end
    AveActLR = AveActLR/dI;
    AveInhLR = AveInhLR/dI;
    Conc3DDiff = AveActLR - AveInhLR;
    
    

    
    
    
    if mod(nstep,50) == 0
        CountStep = CountStep + 1;
        
        Struct3DHF(CountStep).AveActLR = AveActLR;
        Struct3DHF(CountStep).AveInhLR = AveInhLR;
        Struct3DHF(CountStep).h = h;
        Struct3DHF(CountStep).HFTime = HFTime;
        Struct3DHF(CountStep).Time = nstep*dt;
    
    %hmin, HFColorStr = 0; hmax, HFColorStr = 1;
    %HFColorStr = (hmin - h)/(hmin - hmax);
    HFColorStr =  max(0,floor(ActLR-InhLR) + 2);
    %HFColorStr =  max(0,floor(10-InhLR) );
        
    figure(1)
    clf
    set(gca,'FontSize',15,'FontWeight','bold');
    hold on

    for nxHF = 1:1:NX+1
        for nyHF = 1:1:NY+1
            
            
            if nyHF >= NY_Sq_B && nyHF <= NY_Sq_T && nxHF >= NX_Sq_L && nxHF <= NX_Sq_R
                 
                plot((nxHF-1)*dx, (nyHF-1)*dy, 'k.', 'MarkerSize',20)
            
            elseif HFColorStr(nxHF,nyHF) == 0
                
                if nxHF == NX+1 && nyHF == NY+1
                    HFColor = [1-HFColorStr(nxHF,nyHF), 1-HFColorStr(nxHF,nyHF),1];
                    plot((nxHF-1)*dx, (nyHF-1)*dy, '.', 'Color',HFColor,'MarkerSize',20)
                elseif nxHF == 1 && nyHF == 1
                    HFColor = [1-HFColorStr(nxHF,nyHF), 1-HFColorStr(nxHF,nyHF),1];
                    plot((nxHF-1)*dx, (nyHF-1)*dy, '.', 'Color',HFColor,'MarkerSize',20)
                    
                end
                   

                
            else
               % HFColor = [1-HFColorStr(nxHF,nyHF), 1-HFColorStr(nxHF,nyHF),1];
               % plot((nxHF-1)*dx, (nyHF-1)*dy, '.', 'Color',HFColor,'MarkerSize',20)
               
                plot((nxHF-1)*dx, (nyHF-1)*dy, '.', 'Color',cm(min(HFColorStr(nxHF,nyHF),8),:),'MarkerSize',20)
                
            end
            
            
        end
    end
    
    xlim([0,X])
    ylim([0,Y])
    
    
    axis equal
    
    
    StrTitle = sprintf('Time = %g', nstep*dt);
    title(StrTitle)

    hold off
    
  %  figure(2)
  %  clf
  %  PlotX = 0:dx:X ;
  %  %PlotY = 0:1:NY ;
  %  plot(PlotX,AveActLR(:,30),'g-','LineWidth',2)
  %  %plot(PlotY,AveActLR(10,:), 'g-','LineWidth',2)
  %  hold on
  %  plot(PlotX,AveInhLR(:,30),'r-','LineWidth',2)
  %  %plot(PlotY,AveInhLR(10,:),'r-','LineWidth',2)
  %  plot(PlotX,-h(:,30),'-','LineWidth',2)
  %  %plot(PlotY, -h(10,:),'-','LineWidth',2)
  %  hold off
  %  axis([0,X,0,15])
    
  %  figure(3)
  %  clf
  %  PlotZ = 0:1:NZ;
  %  plot(PlotZ*dz,TestActL,'g-','LineWidth',2)
  %  hold on
  %  plot(PlotZ*dz,TestInhL,'r-','LineWidth',2)
  %  TestH = -h(10,1) + (0:10:100)*1i;
  %  plot(real(TestH),imag(TestH),'-','LineWidth',2)
  %  ylim([0,100]) 
  %  hold off

    end
    
    
    
    
end


 



 
    














